%%Uhlman Main Program
% Feb 2 2022
% Todd Fallesen, Calm Facility, Francis Crick Institute

%% Steps
% 1.) Get directory where the images are
% 2.) Get a list of the files in the directory
% 3.) Loop over all images (image loop)
% 4.) In image loop, set the short side as the x-axis, (columns)
% 5.) In the image loop, for each image, loop over all rows and measure the
% intensity over the columns
% 6.) Fit the instensity across each row with a Double Gaussian
% 7.) Export the fitting parameters for each row so we can do math on them
% 8.) Save the exported parameters

%% Data
% For the data, make a structure that will have the rotated image, the
% gaussian fits and their parameters




%%Want to mask the image by the binary image +2 pixels, and fit that!



%Steps 1 and 2

clear all                   %clear all variables
tic                         %start the timer
path = "/camp/home/fallest/working/fallest/Projects/Frank_Uhlman/Proper_Tifs/1_tif/output_selected";  %default path, makes searching a bit faster
warning('off');             %turn off the warnings from Matlab---
extension = "tif";          %give the file extension of the images

outname= "Selected_data";   %This is the name we will use for the saved data structure from image processing 

[input_folder, file_names] = file_importer(path, extension); %get the image directory and the files in the input directory, 
                                                            %returns string
                                                            %for input
                                                            %folder, cell
                                                            %array for
                                                            %files     

file_struct.Filename = [];  %initialize a data structure 
file_struct.ImData = [];
                                                            
%Step 3:

disp_string = strcat("There are ", num2str(length(file_names)), " files");  %Display a message showing how many files there are to the terminal. Just info for the users
disp(disp_string);



%%Main loop, loops over every file in the file list, and does step 3-7 on
%%them

for i=1:length(file_names)
    disp_string_loop = strcat("On file ", num2str(i), " of ", num2str(length(file_names)));   %just giving us a nice readout of where we are in the process
    disp(disp_string_loop);
       
    file = strcat(input_folder, filesep, file_names{i});                                       %construct the image path from the input folder and filename for this iteration
    
    file_struct(i).Filename = file;                                                            %save the filename to the data structure for reference

    imdata = imread(file);                                                                     %Read in the image data for the file of this iteration
    
    
    % Make a binary mask of the image, fill the holes in the binary mask,
    % keep only the biggest object in the binary mask (which should be the
    % chromosome).
    bw_filled = imfill(imbinarize(imdata),'holes');                         %binarize the image, and fill holes
    bw_filled = imfill(bw_filled, 'holes');                                 %fill the holes a second iteration
    bw_filled = bwareafilt(bw_filled, 1);                                   %keep only the biggest blob
    ROI = bwconvhull(bw_filled);                                            %get the roi of the blob
    
    stats = regionprops(ROI, 'BoundingBox','MajorAxisLength','MinorAxisLength');  %get the properties of the blob to be passed to matrix_rotator
    
    %Matrix rotator is a function Todd made that rotates the image if the
    %long side is the rows and the short side the columns (i.e. makes sure
    %the image is portrait, not landscape). It also checks to see if the
    %image has a top row with intensity values, which will only happen if
    %the image is properly rotated.  The height and width, if rotated, will
    %swap in the matrix rotator function
    
    [imdata, bw_filled, width, height, rotated] = matrix_rotator(stats, imdata, bw_filled);
    
    file_struct(i).ROI_width=width;   	%save the mask properties out to the structure
    file_struct(i).ROI_height=height;
    file_struct(i).rotated=rotated;
    

           file_struct(i).ImData =imdata; % save the image data to the data structure
           file_struct(i).ImBW = bw_filled; %make a binary image
           file_struct(i).ImOrig = imdata;
           

%
    %Mask the original image data (rotated if need be!) before fitting.
    %This way we zero out the pixels that are past the edge of the
    %chromosomes, for easier fitting
    
    %First we expand the binary mask by 2 pixels
    
    se = strel('disk',2,8);                                                                 %generate the 2 pixels disc structure element, with 8 verticies
    file_struct(i).ImData = image_masker(file_struct(i).ImOrig, file_struct(i).ImBW, se);   %mask the rotated original image with the expanded mask, and save it as ImData
    
    %find the type of the image
    
    if isa(file_struct(i).ImOrig, 'uint16')  %asks if the image is 16 bit
        bits=16;
    elseif isa(file_struct(i).ImOrig, 'uint8') %asks if the image is 8 bit
        bits=8;
    else
        bits=0;   %if the image is anything other than 8 or 16 bit, we don't normalize.  We'll just divide it by 2^0 which, of course, is 1.
    end
    
    file_struct(i).ImData_NonNormal = file_struct(i).ImData; %save the non-normalized data
    file_struct(i).ImData = double(file_struct(i).ImData)./2^bits;   %normalize the image by the bits
        
    
    
    imdata = file_struct(i).ImData;
    
    x = 1:size(imdata,2); %the x positions, this doesn't vary per image image, so doesn't need to be in the nested loop

    %%
    data_struct(size(imdata,1)).Filename=file;    %make a new temporary data structure to save the curve fitting data.  Initalize it using the filename. Could initilze it with anything.
    
    %Loop over all the rows in the image, to read the intensities and to do
    %the double gaussian fitting.  (Steps 5 and 6).
    
    for j=1:size(imdata,1)              %iterate over all rows
        data_struct(j).Filename=file;   %save the file name to the rows
        
%         disp("break")
%         disp(strcat("i is: ", num2str(i)));  %troubleshooting code to see where you are
%         disp(strcat("j is: ", num2str(j)));
%         disp("break")
        try
            intensity = imdata(j,:);                            %get the intensity for each row, as a double for fitting
            [fitresult, gof] = DoubleGaussFit(x, intensity, true);    %Run the fitting algorithm.  Do check this if there needs to be changes to the fit. The final argument is true to exclude zeros in intensity while fitting
            data_struct(j).Fitresult=fitresult;                 %save the fit results
            data_struct(j).GoF=gof;                             %save the goodness of fit
        catch
            data_struct(j).Fitresult=0;                         %save a zero as the fit result if no fit can be found
            data_struct(j).GoF=0;
        end
        
        try
            data_struct(j).Binary_Width=nnz(bw_filled(j,:));    %get the width, if it exists
        catch
            disp("There was no binary image")
        end
    
    end
    
    file_struct(i).FitParams = data_struct;                 %save the temporary data structure to the main data structure
    clear data_struct imdata;                               %clear the temporary variables
    
    
    
end
    
    struct_name = strcat(outname, '_data_struct.mat');  %generate a name for the data structure
    save(struct_name, "file_struct");                   %save the data structure
toc

